package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.stub.ncshipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;


/**
 * Sample code for the GetNonContractShipmentReceipt Canada Post service.
 * <p>
 * The GetNonContractShipmentReceipt service is used to retrieve XML details
 * regarding the shipment paid for in the Create Non-Contract Shipment call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */

public class GetNCShipmentReceiptClient {

    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            NCShipmentService service = new NCShipmentService();
            NCShipmentPortType port = service.getNCShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));
    		
        	//create request
            GetNonContractShipmentReceiptRequest request = new GetNonContractShipmentReceiptRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
        	request.setShipmentId("522121393172629625");

        	// Execute Request
        	GetNonContractShipmentReceiptResponse response = port.getNCShipmentReceipt(request);

            // Retrieve values from response object
            if (response.getMessages() == null) {
                NonContractShipmentReceiptType nonContractShipmentReceipt= response.getNonContractShipmentReceipt();
                System.out.println("Shipping Point Name: " + nonContractShipmentReceipt.getShippingPointName());
                System.out.println("Service Code: " + nonContractShipmentReceipt.getServiceCode());
                System.out.println("Merchant Name: " + nonContractShipmentReceipt.getCcReceiptDetails().getMerchantName());
                System.out.println("Timestamp: " + nonContractShipmentReceipt.getCcReceiptDetails().getAuthTimestamp());
                System.out.println("Charge Amount: " + nonContractShipmentReceipt.getCcReceiptDetails().getChargeAmount());
                System.out.println("Currency: " + nonContractShipmentReceipt.getCcReceiptDetails().getCurrency());
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
