package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.stub.ncshipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;


/**
 * Sample code for the GetNonContractShipments Canada Post service.
 * <p>
 * The GetNonContractShipments service is used to retrieve the shipment IDs 
 * for previously created shipments.
  * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */

public class GetNCShipmentsClient {

    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            NCShipmentService service = new NCShipmentService();
            NCShipmentPortType port = service.getNCShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));
    		
        	//create request
            GetNonContractShipmentsRequest request = new GetNonContractShipmentsRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setFrom(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-04-22T00:00:00"));
            request.setTo(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-05-22T00:00:00"));


        	// Execute Request
            GetNonContractShipmentsResponse response = port.getNCShipments(request);

            // Retrieve values from response object
            if (response.getMessages() == null) {
                NonContractShipmentsType nonContractShipmentsType = response.getNonContractShipments();
                if(nonContractShipmentsType.getShipmentId() != null && ! nonContractShipmentsType.getShipmentId().isEmpty()){
                    for(String id : nonContractShipmentsType.getShipmentId()){
                        System.out.println("Shipment Id: " + id);
                    }
                } else {
                    System.out.println("No shipments returned.");
                }

            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
