package ca.canadapost.cpcdp.returns;


import java.math.BigDecimal;
import java.util.Map;
import ca.canadapost.cpcdp.returns.stub.authreturn.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import ca.canadapost.cpcdp.returns.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

/**
 * Sample code for the CreateAuthorizedReturn Canada Post service.
 * <p>
 * The CreateAuthorizedReturn service is used to create a return shipping label 
 * when the sender address and approximate package weight are known. This type of 
 * label is typically emailed through a self-serve or service agent process to the
 * individual returning the item.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class CreateAuthReturnClient {

    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
           	// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Returns_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            AuthReturnService service = new AuthReturnService();
            AuthReturnPortType port = service.getAuthReturnPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/authreturn/v2");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));


            // Create Request
            DomesticAddressDetailsType domesticAddressDetailsType = new DomesticAddressDetailsType();
            domesticAddressDetailsType.setAddressLine1("502 MAIN ST N");
            domesticAddressDetailsType.setCity("MONTREAL");
            domesticAddressDetailsType.setProvince("QC");
            domesticAddressDetailsType.setPostalCode("H2B1A0");

            DomesticAddressDetailsType domesticAddressDetailsType2 = new DomesticAddressDetailsType();
            domesticAddressDetailsType2.setAddressLine1("123 Postal Drive");
            domesticAddressDetailsType2.setCity("Ottawa");
            domesticAddressDetailsType2.setProvince("ON");
            domesticAddressDetailsType2.setPostalCode("K1P5Z9");

            ReturnerType returnerType = new ReturnerType();
            returnerType.setName("Bulma");
            returnerType.setCompany("Capsule Corp.");
            returnerType.setDomesticAddress(domesticAddressDetailsType);

            ReceiverType receiverType = new ReceiverType();
            receiverType.setName("John Doe");
            receiverType.setCompany("ACME Corp");
            receiverType.setDomesticAddress(domesticAddressDetailsType2);

            ParcelCharacteristicsType parcelCharacteristicsType = new ParcelCharacteristicsType();
            parcelCharacteristicsType.setWeight(new BigDecimal(15));

            PrintPreferencesType printPreferencesType = new PrintPreferencesType();
            printPreferencesType.setEncoding("PDF");

            AuthSettlementInfoType authSettlementInfoType = new AuthSettlementInfoType();
            authSettlementInfoType.setContractId("0040662505");

            AuthorizedReturnType authorizedReturnType = new AuthorizedReturnType();
            authorizedReturnType.setServiceCode("DOM.EP");
            authorizedReturnType.setReturner(returnerType);
            authorizedReturnType.setReceiver(receiverType);
            authorizedReturnType.setParcelCharacteristics(parcelCharacteristicsType);
            authorizedReturnType.setPrintPreferences(printPreferencesType);
            authorizedReturnType.setSettlementInfo(authSettlementInfoType);

            CreateAuthorizedReturnRequest request = new CreateAuthorizedReturnRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setAuthorizedReturn(authorizedReturnType);

            CreateAuthorizedReturnResponse response = port.createAuthorizedReturn(request);

            // Retrieve values from response object
            if (response.getMessages() == null) {
            	System.out.println("Tracking Pin: " + response.getAuthorizedReturnInfo().getTrackingPin());
            	for ( ArtifactType artifactType : response.getAuthorizedReturnInfo().getArtifacts().getArtifact()) {
	                System.out.println("Artifact Id: " + artifactType.getArtifactId());
            	}
            } else {
            	// Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
