package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.customerinfo.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetCustomerInformation Canada Post service.
 * <p>
 * The GetCustomerInformation service is used to retrieve details on allowed
 * payers, methods of payment and other information about a customer to determine
 * the proper customer number for charging shipping costs.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetCustomerInformationClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            CustomerInfoService service = new CustomerInfoService();
            CustomerInfoPortType port = service.getCustomerInfoPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/customerinfo");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetCustomerInformationRequest request = new GetCustomerInformationRequest();
            request.setLocale("FR");
            request.setCustomerNumber(mailedBy);

            //execute request
            GetCustomerInformationResponse response = port.getCustomerInformation(request);
            if(response.getMessages() == null){
                System.out.println("Customer Number: " + response.getCustomer().getCustomerNumber());
                System.out.println();

                System.out.println("Contract Ids:");
                for ( int i = 0; i < response.getCustomer().getContracts().getContractId().size(); i++) {
                    System.out.println("- " + response.getCustomer().getContracts().getContractId().get(i));
                }
                System.out.println();

                System.out.println("Payers:");
                for ( int i = 0; i < response.getCustomer().getAuthorizedPayers().getPayer().size(); i++) {
                    System.out.println("- Customer Number: " + response.getCustomer().getAuthorizedPayers().getPayer().get(i).getPayerNumber());
                    for ( int j = 0; j < response.getCustomer().getAuthorizedPayers().getPayer().get(i).getMethodsOfPayment().getMethodOfPayment().size(); j++) {
                        if ( j == 0 ) {
                            System.out.println("  Payment Methods:");
                        }
                        System.out.println("  - " + response.getCustomer().getAuthorizedPayers().getPayer().get(i).getMethodsOfPayment().getMethodOfPayment().get(j));
                    }
                    System.out.println();
                }

                if (response.getCustomer().getMobos() != null) {
                    for ( int i = 0; i < response.getCustomer().getMobos().getMobo().size(); i++) {
                        System.out.println("Mobo: " + response.getCustomer().getMobos().getMobo().get(i));
                    }
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
